// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =					 All rights reserved.
// ====================================================================================
// 
//  __	    __	    __
//  \ \	   /  \	   / /   [File name   ] video_top.v
//   \ \  / /\ \  / /	 [Description ] Video demo
//	  \ \/ /  \ \/ /	 [Timestamp   ] Friday May 26 14:00:30 2019
//	   \  /	   \  /	     [version	  ] 1.0.0
//	    \/	    \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:	|  Author	| Mod. Date	| Changes Made:
// ----------------------------------------------------------------------------------
// V1.0	| Caojie	 | 11/22/19	 | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========

	`define  	SDI_HD_720P60
// 	`define    	SDI_HD_720P50
//	`define		SDI_HD_1080P30
//	`define		SDI_HD_1080P25
//	`define		SDI_HD_1080P24
//	`define		SDI_3G_1080P60
// 	`define    	SDI_3G_1080P50

module video_top
(
	input             I_clk           , //50Mhz 
	input             I_key1          ,
	output     [3:0]  O_led           ,  
	inout	          IO_adv7513_scl  ,
	inout	          IO_adv7513_sda  ,
	output            O_adv7513_clk    ,
	output            O_adv7513_vs    ,
	output            O_adv7513_hs    ,
	output            O_adv7513_de    ,
	output     [23:0] O_adv7513_data     
);

//======================================
localparam PWIDTH = 20;

//------------------------------------
wire lane3_pcs_tx_o_fabric_clk;
wire lane3_pcs_rx_o_fabric_clk;


wire [87:0] rx_pdata;

wire lane0_rx_if_fifo_aempty;



//==================================================
wire        rst_n1;
wire        rst_n2;


//--------------------------
reg  [31:0] run_cnt;
wire        running;

reg  [31:0] run_cnt1;
wire        running1;
//------------------------------------------
//ADV7513 I2c
wire         TX_EN_7513;
wire [2:0]   WADDR_7513;
wire [7:0]   WDATA_7513;
wire         RX_EN_7513;
wire [2:0]   RADDR_7513;
wire [7:0]   RDATA_7513;

wire         clk_pll;
wire         sclk;
wire         clk_osc;

//==============================================
`ifdef SDI_HD_720P60
	wire [2:0]  I_rate      = 3'd1    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
	wire [15:0] I_hres      = 16'd1280;
	wire [15:0] I_vres      = 16'd720 ;
	wire [2:0]  I_ver_fre   = 3'd0    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	wire        I_interlace = 1'b1    ;//0:interlaced，1:progressive
	wire        I_color     = 1'b0    ;//0:YC，1:RGB
	wire        I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
	wire        I_pixbit    = 1'b0    ;//0:10bit，1:12bit
	wire [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd1650;//hor total time 
    wire [15:0] I_h_sync    = 16'd40  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd220 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1280;//hor resolution
    wire [15:0] I_v_total   = 16'd750 ;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd20  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd720 ;//ver resolution 
`endif

`ifdef SDI_HD_720P50
	wire [2:0]  I_rate      = 3'd1    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
	wire [15:0] I_hres      = 16'd1280;
	wire [15:0] I_vres      = 16'd720 ;
	wire [2:0]  I_ver_fre   = 3'd1    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	wire        I_interlace = 1'b1    ;//0:interlaced，1:progressive
	wire        I_color     = 1'b0    ;//0:YC，1:RGB
	wire        I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
	wire        I_pixbit    = 1'b0    ;//0:10bit，1:12bit
	wire [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd1980;//hor total time 
    wire [15:0] I_h_sync    = 16'd40  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd220 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1280;//hor resolution
    wire [15:0] I_v_total   = 16'd750 ;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd20  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd720 ;//ver resolution 
`endif

`ifdef SDI_HD_1080P30
    reg  [2:0]  I_rate      = 3'd1    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
    reg  [15:0] I_hres      = 16'd1920;
    reg  [15:0] I_vres      = 16'd1080;
    reg  [2:0]  I_ver_fre   = 3'd2    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
    reg         I_interlace = 1'b1    ;//0:interlaced，1:progressive
    reg         I_color     = 1'b0    ;//0:YC，1:RGB
    reg         I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
    reg         I_pixbit    = 1'b0    ;//0:10bit，1:12bit
    reg  [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd2200;//hor total time 
    wire [15:0] I_h_sync    = 16'd44  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd148 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1920;//hor resolution
    wire [15:0] I_v_total   = 16'd1125;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd36  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd1080;//ver resolution 	
`endif

`ifdef SDI_HD_1080P25
	wire [2:0]  I_rate      = 3'd1    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
	wire [15:0] I_hres      = 16'd1920;
	wire [15:0] I_vres      = 16'd1080;
	wire [2:0]  I_ver_fre   = 3'd3    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	wire        I_interlace = 1'b1    ;//0:interlaced，1:progressive
	wire        I_color     = 1'b0    ;//0:YC，1:RGB
	wire        I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
	wire        I_pixbit    = 1'b0    ;//0:10bit，1:12bit
	wire [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd2640;//hor total time 
    wire [15:0] I_h_sync    = 16'd44  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd148 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1920;//hor resolution
    wire [15:0] I_v_total   = 16'd1125;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd36  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd1080;//ver resolution 
`endif

`ifdef SDI_HD_1080P24
	wire [2:0]  I_rate      = 3'd1    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
	wire [15:0] I_hres      = 16'd1920;
	wire [15:0] I_vres      = 16'd1080;
	wire [2:0]  I_ver_fre   = 3'd4    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	wire        I_interlace = 1'b1    ;//0:interlaced，1:progressive
	wire        I_color     = 1'b0    ;//0:YC，1:RGB
	wire        I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
	wire        I_pixbit    = 1'b0    ;//0:10bit，1:12bit
	wire [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd2750;//hor total time 
    wire [15:0] I_h_sync    = 16'd44  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd148 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1920;//hor resolution
    wire [15:0] I_v_total   = 16'd1125;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd36  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd1080;//ver resolution 
`endif

`ifdef SDI_3G_1080P60
	wire [2:0]  I_rate      = 3'd2    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
	wire [15:0] I_hres      = 16'd1920;
	wire [15:0] I_vres      = 16'd1080;
	wire [2:0]  I_ver_fre   = 3'd0    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	wire        I_interlace = 1'b1    ;//0:interlaced，1:progressive
	wire        I_color     = 1'b0    ;//0:YC，1:RGB
	wire        I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
	wire        I_pixbit    = 1'b0    ;//0:10bit，1:12bit
	wire [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd2200;//hor total time 
    wire [15:0] I_h_sync    = 16'd44  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd148 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1920;//hor resolution
    wire [15:0] I_v_total   = 16'd1125;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd36  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd1080;//ver resolution 
`endif

`ifdef SDI_3G_1080P50
	wire [2:0]  I_rate      = 3'd2    ;//0:SD-SDI, 1:HD-SDI, 2:3G-SDI, 3:6G-SDI
	wire [15:0] I_hres      = 16'd1920;
	wire [15:0] I_vres      = 16'd1080;
	wire [2:0]  I_ver_fre   = 3'd1    ;//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	wire        I_interlace = 1'b1    ;//0:interlaced，1:progressive
	wire        I_color     = 1'b0    ;//0:YC，1:RGB
	wire        I_mfactor   = 1'b0    ;//0:M=1，1:M=1.001
	wire        I_pixbit    = 1'b0    ;//0:10bit，1:12bit
	wire [1:0]  I_pixstruc  = 2'b00   ;//2'b00:4:2:2，2'b10:4:4:4，2'b11:4:4:4:4
	
	wire [15:0] I_h_total   = 16'd2640;//hor total time 
    wire [15:0] I_h_sync    = 16'd44  ;//hor sync time
    wire [15:0] I_h_bporch  = 16'd148 ;//hor back porch
    wire [15:0] I_h_res     = 16'd1920;//hor resolution
    wire [15:0] I_v_total   = 16'd1125;//ver total time 
    wire [15:0] I_v_sync    = 16'd5   ;//ver sync time  
    wire [15:0] I_v_bporch  = 16'd36  ;//ver back porch  
    wire [15:0] I_v_res     = 16'd1080;//ver resolution 
`endif
//----------------------
wire        pixel_clock;
wire        drp_clk;

//-------------
//driver
wire [15:0] vs_cnt;
wire [15:0] hs_cnt;
wire        vsync; 
wire        hsync; 
wire        data_valid; 
wire [7:0]  data0_r; 
wire [7:0]  data0_g;
wire [7:0]  data0_b;

reg         vs_r;
reg  [9:0]  cnt_vs;

//----------------------
wire [15:0] vs_cnt_yc;
wire [15:0] hs_cnt_yc;
wire        fld_yc;
wire        vs_yc;  
wire        hs_yc;  
wire        de_yc;
wire [15:0] data_yc;

//---------------------
wire        fld_in;
wire        vs_in;  
wire        hs_in;  
wire        de_in;
wire [19:0] data_in;

//-------------


//-------------
wire        clk_rx;
wire        fld_rx;
wire        vs_rx;  
wire        hs_rx;  
wire        de_rx;
wire [19:0] data_rx;

wire [15:0] rx_hcnt ;
wire [15:0] rx_vcnt ;

reg         vs_rx_gen;
reg         hs_rx_gen;

//---------------
wire        yc422_clk_in ;
wire        yc422_vs_in  ;  
wire        yc422_hs_in  ;  
wire        yc422_de_in  ;
wire [15:0] yc422_data_in;

//--------------
wire        vs_rgb;  
wire        hs_rgb;  
wire        de_rgb;  
wire [7:0]  data_rgb_r;
wire [7:0]  data_rgb_g;
wire [7:0]  data_rgb_b;



Gowin_PLL Gowin_PLL_inst
(
 
    .clkout0(sclk       ), //50MHz
    .clkin  (I_clk      )  
);

//==================================================
key_debounceN #
(
	.DEBCNT1 (32'd50000000 ),//1s
	.DEBCNT2 (32'd100000000) //2s
) 
key_debounceN_inst0
(
	.clk       (sclk   ), 
	.rst_n     (1'b1    ), 
	.key_in    (I_key1  ),//I_rst_n ), 
	.key_n_out1(rst_n1  ),
	.key_n_out2(rst_n2  )
);


//===================================================
//LED灯测试
always @(posedge pixel_clock or negedge rst_n1)
begin
	if(!rst_n1)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd148_500_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd74_250_000) ? 1'b1 : 1'b0;

always @(posedge clk_rx or negedge rst_n1)
begin
	if(!rst_n1)
		run_cnt1 <= 32'd0;
	else if(run_cnt1 >= 32'd148_500_000)
		run_cnt1 <= 32'd0;
	else
		run_cnt1 <= run_cnt1 + 1'b1;
end

assign  running1 = (run_cnt1 < 32'd74_250_000) ? 1'b1 : 1'b0;


assign  O_led[0] = running1;
assign  O_led[1] = running;
assign  O_led[2] = rst_n1;
assign  O_led[3] = rst_n2;
 
//=========================================================================
// ADV7513 I2c 初始化
  adv7513_iic_init 	    adv7513_iic_init_inst0
  (
      .I_CLK              ( sclk                  ),
      .I_RESETN           ( rst_n1                ),//low active
	  .start              ( rst_n2                ),
      .O_TX_EN            ( TX_EN_7513            ),
      .O_WADDR            ( WADDR_7513            ),
      .O_WDATA            ( WDATA_7513            ),
      .O_RX_EN            ( RX_EN_7513            ),
      .O_RADDR            ( RADDR_7513            ),
      .I_RDATA            ( RDATA_7513            ),
      .cstate_flag        (                       ),
      .error_flag         (                       )
  );



  I2C_MASTER_Top        I2C_MASTER_Top_inst0
  (
      .I_CLK              ( sclk                 ),
      .I_RESETN           ( rst_n2                 ),//low active
      .I_TX_EN            ( TX_EN_7513            ),
      .I_WADDR            ( WADDR_7513            ),
      .I_WDATA            ( WDATA_7513            ),
      .I_RX_EN            ( RX_EN_7513            ),
      .I_RADDR            ( RADDR_7513            ),
      .O_RDATA            ( RDATA_7513            ),
      .O_IIC_INT          (                       ),
      .SCL                ( IO_adv7513_scl           ),
      .SDA                ( IO_adv7513_sda           )
  );
  
        
//===================================================



//==============================================

assign pixel_clock = lane3_pcs_tx_o_fabric_clk;
assign drp_clk = sclk;

//===========================================================================
//testpattern
testpattern testpattern_inst
(
	.I_pxl_clk   (pixel_clock        ),//pixel clock
	.I_rst_n     (rst_n1             ),//low active 
	.I_mode      ({1'b0,1'd0,cnt_vs[7]} ),//data select
    .I_sqr_width (16'd60             ),
	.I_single_r  (8'd0               ),
	.I_single_g  (8'd255             ),                  //74.25MHz  //148.5MHz 
	.I_single_b  (8'd0               ),                  //1280x720  //1920x1080   
	.I_h_total   (I_h_total          ),//hor total time  // 16'd1650 // 16'd2200  
	.I_h_sync    (I_h_sync           ),//hor sync time   // 16'd40   // 16'd44    
	.I_h_bporch  (I_h_bporch         ),//hor back porch  // 16'd220  // 16'd148   
	.I_h_res     (I_h_res            ),//hor resolution  // 16'd1280 // 16'd1920  
	.I_v_total   (I_v_total          ),//ver total time  // 16'd750  // 16'd1125   
	.I_v_sync    (I_v_sync           ),//ver sync time   // 16'd5    // 16'd5     
	.I_v_bporch  (I_v_bporch         ),//ver back porch  // 16'd20   // 16'd36    
	.I_v_res     (I_v_res            ),//ver resolution  // 16'd720  // 16'd1080   
	.I_hs_pol    (1'b1               ),//HS polarity , 0:negetive ploarity，1：positive polarity
	.I_vs_pol    (1'b1               ),//VS polarity , 0:negetive ploarity，1：positive polarity
    .O_V_cnt     (vs_cnt             ),
    .O_H_cnt     (hs_cnt             ),
	.O_de        (data_valid         ),   
	.O_hs        (hsync              ),
	.O_vs        (vsync              ),
	.O_data_r    (data0_r            ),   
	.O_data_g    (data0_g            ),
	.O_data_b    (data0_b            )
);

always@(posedge pixel_clock)
begin
	vs_r<=vsync;
end

always@(posedge pixel_clock or negedge rst_n1)
begin
	if(!rst_n1)
		cnt_vs<=10'd0;
	else if(vs_r && !vsync) //vs24 falling edge
		cnt_vs<=cnt_vs+1'b1;
end 

//==============================================
//rgb to yc422
rgb_yc422 rgb_yc422_inst
(
`ifdef SD_SDI
    .pix_clk   (           ),
`else
    .pix_clk   (pixel_clock), 
`endif 
    .rst_n     (rst_n1      ),
    .odd_even  (1'b0       ), //default 0
    .vs_cnt_i  (vs_cnt     ),
    .hs_cnt_i  (hs_cnt     ),
    .fld_i     (1'b0       ),
    .hs_i      (hsync      ),
    .vs_i      (vsync      ),
    .din_valid (data_valid ),      
    .r_in      (data0_r    ),       
    .g_in      (data0_g    ),       
    .b_in      (data0_b    ), 
    .vs_cnt_o  (vs_cnt_yc  ),
    .hs_cnt_o  (hs_cnt_yc  ),
    .dout_valid(de_yc      ), 
    .fld_o     (fld_yc     ),
    .hs_o      (hs_yc      ),
    .vs_o      (vs_yc      ),
    .ycbcr422  (data_yc    )
);

//==============================================
    assign fld_in  = fld_yc;
    assign vs_in   = vs_yc ;  
    assign hs_in   = hs_yc ;  
    assign de_in   = de_yc ;
    assign data_in = {data_yc[7:0],2'd0,data_yc[15:8],2'd0};//{C,Y}
//=================================================
//SDI IP (Scramble Out and Scramble In)
SerDes_Top Serdes_Top_inst
(
	//about sdi 	pma rst	
	.SDI_Top_sdi_pma_rst_n_i  		 		(rst_n1			),
										
	//sdi tx        	            	
	.SDI_Top_sdi_tx_rst_n_i         		(rst_n1			),//low active
	.SDI_Top_sdi_tx_rate_i         			(I_rate			),//0:SD-SDI, 1:HD-SDI, 2:3G-SDI
	.SDI_Top_sdi_tx_hres_i         			(I_hres			),
	.SDI_Top_sdi_tx_vres_i         			(I_vres			),
	.SDI_Top_sdi_tx_ver_fre_i      			(I_ver_fre		),//0:60Hz, 1:50Hz, 2:30Hz, 3:25Hz, 4:24Hz
	.SDI_Top_sdi_tx_interlace_i    			(I_interlace	),//0:interlaced，1:progressive
	.SDI_Top_sdi_tx_color_i        			(I_color		),//0:YC, 1:RGB
	.SDI_Top_sdi_tx_mfactor_i      			(I_mfactor		),//0:M=1, 1:M=1.001
	.SDI_Top_sdi_tx_pixbit_i       			(I_pixbit		),//0:10bit, 1:12bit
	.SDI_Top_sdi_tx_pixstruc_i     			(I_pixstruc		),//2'b00:4:2:2, 2'b10:4:4:4, 2'b11:4:4:4:4
	.SDI_Top_sdi_tx_fld_i         			(fld_in			),
	.SDI_Top_sdi_tx_vs_i          			(vs_in			),//positive polarity
	.SDI_Top_sdi_tx_hs_i          			(hs_in			),//positive polarity
	.SDI_Top_sdi_tx_de_i          			(de_in			),
	.SDI_Top_sdi_tx_data_i        			(data_in		),//data in, BT656:{10'd0,10bit}, YC422_10b:{C,Y}, other maps:{10bit,10bit}
	.SDI_Top_sdi_tx_half_clk_o				(lane3_pcs_tx_o_fabric_clk				),//serdes output half clock, except BT656
										
	//sdi rx        	            	
	.SDI_Top_sdi_rx_rst_n_i         		(rst_n1			),//low active
	.SDI_Top_sdi_rx_drp_clk_i				(drp_clk		),//<=50MHz 
	.SDI_Top_sdi_rx_fix_rate_en_i      	 	(1'b1			),
	.SDI_Top_sdi_rx_fix_rate_i         		(I_rate			),//only for fix rate //0:SD-SDI, 1:HD-SDI, 2:3G-SDI
	.SDI_Top_sdi_rx_fix_mfactor_i      		(I_mfactor		),//only for fix rate //0:M=1，1:M=1.001
	.SDI_Top_sdi_rx_hres_o             	 	(				),//horizontal resolution
	.SDI_Top_sdi_rx_vres_o             	 	(				),//vertical resolution
	.SDI_Top_sdi_rx_interlace_o        	 	(				),//0:interlaced, 1:progressive
	.SDI_Top_sdi_rx_rate_o               	(				),//0:SD-SDI, 1:HD-SDI, 2:3G-SDI
	.SDI_Top_sdi_rx_mod_o               	(				),//0:148.5M,  1:74.25M,  2:148.5M/1.001, 3:74.25M/1.001 
	.SDI_Top_sdi_rx_hcnt_o       			(rx_hcnt		),
	.SDI_Top_sdi_rx_vcnt_o       			(rx_vcnt		),
	.SDI_Top_sdi_rx_clk_o      				(clk_rx			),
	.SDI_Top_sdi_rx_f_o         			(fld_rx			),
	.SDI_Top_sdi_rx_v_o         			(vs_rx			),
	.SDI_Top_sdi_rx_h_o         			(hs_rx			),
	.SDI_Top_sdi_rx_de_o        			(de_rx			),
	.SDI_Top_sdi_rx_data20_o    			(data_rx		) //{C,Y}
);


//=================================================
always@(posedge clk_rx or negedge rst_n1)
begin
    if(!rst_n1)
        vs_rx_gen <= 1'b0;
    else if(rx_vcnt>=1 && rx_vcnt<=5)
        vs_rx_gen <= 1'b1;
    else
        vs_rx_gen <= 1'b0;
end

always@(posedge clk_rx or negedge rst_n1)
begin
    if(!rst_n1)
        hs_rx_gen <= 1'b0;
    else if(rx_hcnt>=10 && rx_hcnt<=72)
        hs_rx_gen <= 1'b1;
    else
        hs_rx_gen <= 1'b0;
end

    assign yc422_clk_in  = clk_rx    ; 
    assign yc422_vs_in   = vs_rx_gen ;//vs_rx     ;  
    assign yc422_hs_in   = hs_rx_gen ;//hs_rx     ;  
    assign yc422_de_in   = de_rx     ;
    assign yc422_data_in = {data_rx[9:2],data_rx[19:12]};

//==============================================
//yc422 to rgb
yc422_rgb yc422_rgb_inst
(
    .rst_ni          (rst_n1        ),          
    .pxclk_i         (yc422_clk_in ),        
    .cb_cr_first_en_i(1'b0         ),//default 0
    .ycbcr422_vs_i   (yc422_vs_in  ),   
    .ycbcr422_hs_i   (yc422_hs_in  ),   
    .ycbcr422_de_i   (yc422_de_in  ),   
    .ycbcr422_data_i (yc422_data_in), 
    .rgb_vs_o        (vs_rgb       ),        
    .rgb_hs_o        (hs_rgb       ),        
    .rgb_de_o        (de_rgb       ),       //---- delay 6 clock     
    .rgb_r_o         (data_rgb_r   ),         
    .rgb_g_o         (data_rgb_g   ),         
    .rgb_b_o         (data_rgb_b   )         
);

	assign O_adv7513_data 	 = {data_rgb_r,data_rgb_g,data_rgb_b};//{r,g,b}
	assign O_adv7513_vs      = vs_rgb;
	assign O_adv7513_hs      = hs_rgb;
	assign O_adv7513_de 	 = de_rgb;
	assign O_adv7513_clk     = yc422_clk_in;

endmodule